@@ -45,7 +45,7 @@ class TestProp2PO:
         print(pofile)
         return len(pofile.units) - 1
 
-    def test_simpleentry(self):
+    def test_simpleentry(self) -> None:
         """Checks that a simple properties entry converts properly to a po entry."""
         propsource = "SAVEENTRY=Save file\n"
         pofile = self.prop2po(propsource)
@@ -53,7 +53,7 @@ class TestProp2PO:
         assert pounit.source == "Save file"
         assert pounit.target == ""
 
-    def test_convertprop(self):
+    def test_convertprop(self) -> None:
         """Checks that the convertprop function is working."""
         propsource = "SAVEENTRY=Save file\n"
         posource = self.convertprop(propsource)
@@ -62,7 +62,7 @@ class TestProp2PO:
         assert pounit.source == "Save file"
         assert pounit.target == ""
 
-    def test_no_value_entry(self):
+    def test_no_value_entry(self) -> None:
         """Checks that a properties entry without value is converted."""
         propsource = "KEY = \n"
         pofile = self.prop2po(propsource)
@@ -71,7 +71,7 @@ class TestProp2PO:
         assert pounit.source == ""
         assert pounit.target == ""
 
-    def test_no_separator_entry(self):
+    def test_no_separator_entry(self) -> None:
         """Checks that a properties entry without separator is converted."""
         propsource = "KEY\n"
         pofile = self.prop2po(propsource)
@@ -80,7 +80,17 @@ class TestProp2PO:
         assert pounit.source == ""
         assert pounit.target == ""
 
-    def test_tab_at_end_of_string(self):
+    def test_value_no_key_entry(self) -> None:
+        """Checks that a properties entry with value but no key is converted."""
+        propsource = "=value\n"
+        pofile = self.prop2po(propsource)
+        pounit = self.singleelement(pofile)
+        # Empty key should use special <empty> marker for location
+        assert pounit.getlocations() == ["<empty>"]
+        assert pounit.source == "value"
+        assert pounit.target == ""
+
+    def test_tab_at_end_of_string(self) -> None:
         """Check that we preserve tabs at the end of a string."""
         propsource = r"TAB_AT_END=This sentence has a tab at the end.\t"
         pofile = self.prop2po(propsource)
@@ -111,7 +121,7 @@ class TestProp2PO:
         pounit = self.singleelement(pofile)
         assert pounit.source == "This sentence will keep its 4 spaces at the end.\\    "
 
-    def test_tab_at_start_of_value(self):
+    def test_tab_at_start_of_value(self) -> None:
         """Check that tabs in a property are ignored where appropriate."""
         propsource = r"property	=	value"
         pofile = self.prop2po(propsource)
@@ -119,7 +129,7 @@ class TestProp2PO:
         assert pounit.getlocations()[0] == "property"
         assert pounit.source == "value"
 
-    def test_unicode(self):
+    def test_unicode(self) -> None:
         """Checks that unicode entries convert properly."""
         unistring = r"Norsk bokm\u00E5l"
         propsource = f"nb = {unistring}\n"
@@ -129,7 +139,7 @@ class TestProp2PO:
         print(repr(pounit.source))
         assert pounit.source == "Norsk bokm\u00e5l"
 
-    def test_multiline_escaping(self):
+    def test_multiline_escaping(self) -> None:
         """Checks that multiline entries can be parsed."""
         propsource = r"""5093=Unable to connect to your IMAP server. You may have exceeded the maximum number \
 of connections to this server. If so, use the Advanced IMAP Server Settings dialog to \
@@ -138,7 +148,7 @@ reduce the number of cached connections."""
         print(repr(pofile.units[1].target))
         assert self.countelements(pofile) == 1
 
-    def test_comments(self):
+    def test_comments(self) -> None:
         """Test to ensure that we take comments from .properties and place them in .po."""
         propsource = """# Comment
 prefPanel-smime=Security"""
@@ -146,7 +156,7 @@ prefPanel-smime=Security"""
         pounit = self.singleelement(pofile)
         assert pounit.getnotes("developer") == "Comment"
 
-    def test_multiline_comments(self):
+    def test_multiline_comments(self) -> None:
         """Test to ensure that we handle multiline comments well."""
         propsource = """# Comment
 # commentary 2
@@ -162,7 +172,7 @@ prefPanel-smime=
         pounit = self.singleelement(pofile)
         assert pounit.getnotes("developer") == "# @name GENERIC_ERROR\n# @loc none"
 
-    def test_folding_accesskeys(self):
+    def test_folding_accesskeys(self) -> None:
         """Check that we can fold various accesskeys into their associated label (bug #115)."""
         propsource = r"""cmd_addEngine.label = Add Engines...
 cmd_addEngine.accesskey = A"""
@@ -170,7 +180,7 @@ cmd_addEngine.accesskey = A"""
         pounit = self.singleelement(pofile)
         assert pounit.source == "&Add Engines..."
 
-    def test_dont_translate(self):
+    def test_dont_translate(self) -> None:
         """Check that we know how to ignore don't translate instructions in properties files (bug #116)."""
         propsource = """# LOCALIZATION NOTE (dont): DONT_TRANSLATE.
 dont=don't translate me
@@ -179,7 +189,53 @@ do=translate me
         pofile = self.prop2po(propsource)
         assert self.countelements(pofile) == 1
 
-    def test_emptyproperty(self):
+    def test_duplicate_locations_mozilla(self) -> None:
+        """
+        Check that we handle duplicate locations properly in Mozilla properties files.
+        This tests the fix for the Lithuanian Firefox recovery issue where duplicate
+        locations could cause a traceback.
+
+        When there are duplicate keys, locationindex will contain only the first
+        occurrence from the translated file. Both template entries are processed,
+        but both will map to the same translation (the first occurrence).
+        """
+        # Template file with duplicate keys
+        templateprop = """key1=Original value 1
+key1=Original value 2
+key2=Another value
+"""
+        # Translation file with duplicate keys
+        translatedprop = """key1=Translated value 1
+key1=Translated value 2
+key2=Translated value
+"""
+        # Convert using Mozilla personality - should not raise AttributeError
+        pofile = self.prop2po(translatedprop, templateprop, personality="mozilla")
+
+        # We get 3 units because both key1 entries from template are processed
+        # Both map to the first translated value due to locationindex behavior
+        assert self.countelements(pofile) == 3
+
+        # First key1 entry
+        pounit1 = pofile.units[1]
+        assert pounit1.source == "Original value 1"
+        assert pounit1.target == "Translated value 1"
+        assert pounit1.getlocations() == ["key1"]
+
+        # Second key1 entry (duplicate location)
+        pounit2 = pofile.units[2]
+        assert pounit2.source == "Original value 2"
+        # Both key1 entries map to first translated value
+        assert pounit2.target == "Translated value 1"
+        assert pounit2.getlocations() == ["key1"]
+
+        # key2 entry
+        pounit3 = pofile.units[3]
+        assert pounit3.source == "Another value"
+        assert pounit3.target == "Translated value"
+        assert pounit3.getlocations() == ["key2"]
+
+    def test_emptyproperty(self) -> None:
         """Checks that empty property definitions survive into po file, bug 15."""
         for delimiter in ["=", ""]:
             propsource = f"# comment\ncredit{delimiter}"
@@ -191,7 +247,7 @@ do=translate me
             assert b"#. comment" in bytes(pofile)
             assert pounit.source == ""
 
-    def test_emptyproperty_translated(self):
+    def test_emptyproperty_translated(self) -> None:
         """Checks that if we translate an empty property it makes it into the PO."""
         for delimiter in ["=", ""]:
             proptemplate = f"credit{delimiter}"
@@ -204,14 +260,14 @@ do=translate me
             assert pounit.source == ""
             assert pounit.target == "Translators Names"
 
-    def test_newlines_in_value(self):
+    def test_newlines_in_value(self) -> None:
         """Check that we can carry newlines that appear in the property value into the PO."""
         propsource = """prop=\\nvalue\\n\n"""
         pofile = self.prop2po(propsource)
         unit = self.singleelement(pofile)
         assert unit.source == "\nvalue\n"
 
-    def test_header_comments(self):
+    def test_header_comments(self) -> None:
         """Check that we can handle comments not directly associated with a property."""
         propsource = """# Header comment\n\n# Comment\n\nprop=value\n"""
         pofile = self.prop2po(propsource)
@@ -219,7 +275,7 @@ do=translate me
         assert unit.source == "value"
         assert unit.getnotes("developer") == "Comment"
 
-    def test_unassociated_comment_order(self):
+    def test_unassociated_comment_order(self) -> None:
         """Check that we can handle the order of unassociated comments."""
         propsource = """# Header comment\n\n# 1st Unassociated comment\n\n# 2nd Connected comment\nprop=value\n"""
         pofile = self.prop2po(propsource)
@@ -230,7 +286,7 @@ do=translate me
             == "1st Unassociated comment\n\n2nd Connected comment"
         )
 
-    def test_x_header(self):
+    def test_x_header(self) -> None:
         """
         Test that we correctly create the custom header entries
         (accelerators, merge criterion).
@@ -247,7 +303,7 @@ do=translate me
         assert b"X-Accelerator-Marker" not in bytes(outputpo)
         assert b"X-Merge-On" not in bytes(outputpo)
 
-    def test_gaia_plurals(self):
+    def test_gaia_plurals(self) -> None:
         """Test conversion of gaia plural units."""
         propsource = """
 message-multiedit-header={[ plural(n) ]}
@@ -268,7 +324,7 @@ message-multiedit-header[other]={{ n }} selected
         assert not zero_unit.hasplural()
         assert zero_unit.source == "Edit"
 
-    def test_successive_gaia_plurals(self):
+    def test_successive_gaia_plurals(self) -> None:
         """Test conversion of two successive gaia plural units."""
         propsource = """
 message-multiedit-header={[ plural(n) ]}
@@ -305,7 +361,7 @@ message-multiedit-header2[other]={{ n }} selected 2
         assert not zero_unit.hasplural()
         assert zero_unit.source == "Edit"
 
-    def test_duplicate_keys(self):
+    def test_duplicate_keys(self) -> None:
         """Check that we correctly handle duplicate keys."""
         source = """
 key=value
@@ -342,7 +398,7 @@ key2=value
         assert po_unit.source == "value"
         assert po_unit.getlocations() == ["key2"]
 
-    def test_gwt_plurals(self):
+    def test_gwt_plurals(self) -> None:
         """Test conversion of gwt plural units."""
         propsource = """
 message-multiedit-header={0,number} selected
@@ -356,6 +412,48 @@ message-multiedit-header[many]={0,number} selected
         pounit = outputpo.units[-1]
         assert pounit.getlocations() == ["message-multiedit-header"]
 
+    def test_strings_bilingual_simple(self) -> None:
+        """Test that .strings files are treated as bilingual in convertstore mode."""
+        # Simple test case - currently FAILS because source and target are swapped
+        propsource = r""""Source text" = "Translated text";"""
+        # .strings files use UTF-16 encoding
+        inputfile = BytesIO(propsource.encode("utf-16"))
+        inputprop = properties.propfile(inputfile, personality="strings")
+        convertor = prop2po.prop2po(personality="strings")
+        outputpo = convertor.convertstore(inputprop)
+        pounit = self.singleelement(outputpo)
+        # After fix, source should be left side (key) and target should be right side (value)
+        assert pounit.source == "Source text", (
+            f"Expected 'Source text' but got {pounit.source!r}"
+        )
+        assert pounit.target == "Translated text", (
+            f"Expected 'Translated text' but got {pounit.target!r}"
+        )
+
+    def test_strings_bilingual_multiline(self) -> None:
+        """Test multiline .strings bilingual conversion (issue from bug report)."""
+        # This is based on the actual bug report
+        propsource = r"""/* Overwrite of the app folder */
+"A '<AppNameRemovedForPrivacy>' folder has been found in  your Music folder. Unfortunately it seems incomplete.\n\nTo restart <AppNameRemovedForPrivacy> synchronization, trash the ~/Music/<AppNameRemovedForPrivacy> folder and relaunch the application." = "An incomplete <AppNameRemovedForPrivacy> folder has been found in your Music folder.\n\nTo restart, delete the ~/Music/<AppNameRemovedForPrivacy> folder and relaunch the application.";
+"""
+        # .strings files use UTF-16 encoding
+        inputfile = BytesIO(propsource.encode("utf-16"))
+        inputprop = properties.propfile(inputfile, personality="strings")
+        convertor = prop2po.prop2po(personality="strings")
+        outputpo = convertor.convertstore(inputprop)
+        pounit = self.singleelement(outputpo)
+        # The msgid should be the source (left side), not the translation (right side)
+        assert "A '<AppNameRemovedForPrivacy>' folder has been found" in pounit.source
+        assert "Unfortunately it seems incomplete" in pounit.source
+        # The msgstr should be the translation (right side)
+        assert "An incomplete <AppNameRemovedForPrivacy> folder" in pounit.target
+        assert "To restart, delete" in pounit.target
+        # The location should be the key (not URL-encoded full text)
+        locations = pounit.getlocations()
+        assert len(locations) == 1
+        # Location should be a reasonable identifier, not the full multiline text
+        assert "A '<AppNameRemovedForPrivacy>' folder" in locations[0]
+
 
 class TestProp2POCommand(test_convert.TestConvertCommand, TestProp2PO):
     """Tests running actual prop2po commands on files."""
