@@ -1,7 +1,7 @@
 from translate.lang import factory
 
 
-def test_punctranslate():
+def test_punctranslate() -> None:
     """Tests that we can translate punctuation."""
     language = factory.getlanguage("ja")
     assert language.punctranslate("") == ""
@@ -13,7 +13,7 @@ def test_punctranslate():
     assert language.punctranslate("abc efg...") == "abc efg..."
 
 
-def test_sentences():
+def test_sentences() -> None:
     """Tests basic functionality of sentence segmentation."""
     language = factory.getlanguage("ja")
     sentences = language.sentences("")
