@@ -9,7 +9,7 @@ from translate.storage.php import phpunit
 from . import test_base
 
 
-def test_roundtrip_quoting():
+def test_roundtrip_quoting() -> None:
     specials = [
         "Fish & chips",
         "five < six",
@@ -43,7 +43,7 @@ def test_roundtrip_quoting():
 class TestPOUnit(test_base.TestTranslationUnit):
     UnitClass = po.pounit
 
-    def test_istranslatable(self):
+    def test_istranslatable(self) -> None:
         """Tests for the correct behaviour of istranslatable()."""
         unit = self.UnitClass("Message")
         assert unit.istranslatable()
@@ -63,10 +63,10 @@ class TestPOUnit(test_base.TestTranslationUnit):
         unit.makeobsolete()
         assert not unit.istranslatable()
 
-    def test_locations(self):
+    def test_locations(self) -> None:
         """Tests that we can add and retrieve error messages for a unit."""
 
-        def locations_helper(location):
+        def locations_helper(location) -> None:
             unit = self.UnitClass()
             assert len(unit.getlocations()) == 0
             unit.addlocation(location)
@@ -78,14 +78,14 @@ class TestPOUnit(test_base.TestTranslationUnit):
         locations_helper("I am a key")
         locations_helper("unicoḓe key")
 
-    def test_nongettext_location(self):
+    def test_nongettext_location(self) -> None:
         """Test that we correctly handle a non-gettext (file:linenumber) location."""
         u = self.UnitClass("")
         u.addlocation("programming/C/programming.xml:44(para)")
         assert "programming/C/programming.xml:44(para)" in str(u)
         assert "programming/C/programming.xml:44(para)" in u.getlocations()
 
-    def test_adding_empty_note(self):
+    def test_adding_empty_note(self) -> None:
         unit = self.UnitClass("bla")
         print(str(unit))
         assert "#" not in str(unit)
@@ -93,7 +93,7 @@ class TestPOUnit(test_base.TestTranslationUnit):
             unit.addnote(empty_string)
             assert "#" not in str(unit)
 
-    def test_markreview(self):
+    def test_markreview(self) -> None:
         """Tests if we can mark the unit to need review."""
         unit = self.unit
         # We have to explicitly set the target to nothing, otherwise xliff
@@ -117,7 +117,7 @@ class TestPOUnit(test_base.TestTranslationUnit):
         notes = unit.getnotes(origin="translator")
         assert notes.count("Double check spelling.") == 1
 
-    def test_errors(self):
+    def test_errors(self) -> None:
         """Tests that we can add and retrieve error messages for a unit."""
         unit = self.unit
 
@@ -132,7 +132,7 @@ class TestPOUnit(test_base.TestTranslationUnit):
         unit.adderror(errorname="test1", errortext="New error 1.")
         assert unit.geterrors()["test1"] == "New error 1."
 
-    def test_no_plural_settarget(self):
+    def test_no_plural_settarget(self) -> None:
         """Tests that target handling of file with no plural is correct."""
         # plain text, no plural test
         unit = self.UnitClass("Tree")
@@ -152,7 +152,7 @@ class TestPOUnit(test_base.TestTranslationUnit):
             unit.target = ["ki", "ni ki"]
         assert not unit.hasplural()
 
-    def test_wrapping_bug(self):
+    def test_wrapping_bug(self) -> None:
         """This tests for a wrapping bug that existed at some stage."""
         unit = self.UnitClass("")
         message = 'Projeke ya Pootle ka boyona e ho <a href="http://translate.sourceforge.net/">translate.sourceforge.net</a> moo o ka fumanang dintlha ka source code, di mailing list jwalo jwalo.'
@@ -160,14 +160,14 @@ class TestPOUnit(test_base.TestTranslationUnit):
         print(unit.target)
         assert unit.target == message
 
-    def test_extract_msgidcomments_from_text(self):
+    def test_extract_msgidcomments_from_text(self) -> None:
         """Test that KDE style comments are extracted correctly."""
         unit = self.UnitClass("test source")
 
         kdetext = "_: Simple comment\nsimple text"
         assert unit._extract_msgidcomments(kdetext) == "Simple comment"
 
-    def test_isheader(self):
+    def test_isheader(self) -> None:
         """Checks that we deal correctly with headers."""
         unit = self.UnitClass()
         unit.target = "PO-Revision-Date: 2006-02-09 23:33+0200\n"
@@ -177,7 +177,7 @@ class TestPOUnit(test_base.TestTranslationUnit):
         unit.source = "Goeiemôre"
         assert not unit.isheader()
 
-    def test_buildfromunit(self):
+    def test_buildfromunit(self) -> None:
         unit = self.UnitClass("test source")
         unit_copy = self.UnitClass.buildfromunit(unit)
         assert unit is not unit_copy
@@ -239,7 +239,53 @@ class TestPOFile(test_base.TestTranslationStore):
             u.target = u.target
         return bytes(pofile).decode("utf-8")
 
-    def test_context_only(self):
+    def _get_expected_for_gettext_version(self, **version_outputs):
+        """
+        Helper to select expected output based on gettext version.
+
+        Args:
+            version_outputs: Keyword arguments like gettext_0_22="...", gettext_0_23="..."
+
+        Returns the latest gettext version for pypo (Python wrapper follows latest gettext),
+        or selects based on actual libgettextpo version for cpo storage.
+
+        """
+        # Sort versions to find the latest
+        versions = sorted(
+            [
+                (tuple(map(int, k.split("_")[1:])), v)
+                for k, v in version_outputs.items()
+            ],
+            reverse=True,
+        )
+        _latest_version, latest_output = versions[0]
+
+        if issubclass(self.StoreClass, pypo.pofile):
+            # Python wrapper should follow the latest gettext
+            return latest_output
+
+        # Choose matching output depending on gettext version
+        # pylint: disable-next=import-outside-toplevel
+        from translate.storage.cpo import get_libgettextpo_version  # noqa: PLC0415
+
+        actual_version = get_libgettextpo_version()
+
+        # Find the best match
+        for version, output in versions:
+            if actual_version >= version:
+                print(
+                    f"Detected gettext {'.'.join(map(str, version))} or newer ({actual_version})"  # ty:ignore[invalid-argument-type]
+                )
+                return output
+
+        # Fallback to the oldest version
+        oldest_version, oldest_output = versions[-1]
+        print(
+            f"Detected gettext older than {'.'.join(map(str, oldest_version))} ({actual_version})"  # ty:ignore[invalid-argument-type]
+        )
+        return oldest_output
+
+    def test_context_only(self) -> None:
         """Checks that an empty msgid with msgctxt is handled correctly."""
         posource = """msgctxt "CONTEXT"
 msgid ""
@@ -251,7 +297,7 @@ msgstr ""
         # we were not generating output for these at some stage
         assert bytes(pofile)
 
-    def test_simpleentry(self):
+    def test_simpleentry(self) -> None:
         """Checks that a simple po entry is parsed correctly."""
         posource = '#: test.c:100 test.c:101\nmsgid "test"\nmsgstr "rest"\n'
         pofile = self.poparse(posource)
@@ -261,7 +307,7 @@ msgstr ""
         assert thepo.source == "test"
         assert thepo.target == "rest"
 
-    def test_copy(self):
+    def test_copy(self) -> None:
         """Checks that we can copy all the needed PO fields."""
         posource = '''# TRANSLATOR-COMMENTS
 #. AUTOMATIC-COMMENTS
@@ -275,26 +321,26 @@ msgstr "TRANSLATED-STRING"'''
         newunit = oldunit.copy()
         assert newunit == oldunit
 
-    def test_parse_source_string(self):
+    def test_parse_source_string(self) -> None:
         """Parse a string."""
         posource = '#: test.c\nmsgid "test"\nmsgstr "rest"\n'
         pofile = self.poparse(posource)
         assert len(pofile.units) == 1
 
-    def test_parse_file(self):
+    def test_parse_file(self) -> None:
         """Test parsing a real file."""
         posource = '#: test.c\nmsgid "test"\nmsgstr "rest"\n'
         pofile = self.poparse(posource)
         assert len(pofile.units) == 1
 
-    def test_unicode(self):
+    def test_unicode(self) -> None:
         """Check that the po class can handle Unicode characters."""
         posource = 'msgid ""\nmsgstr ""\n"Content-Type: text/plain; charset=UTF-8\\n"\n\n#: test.c\nmsgid "test"\nmsgstr "rest\xe2\x80\xa6"\n'
         pofile = self.poparse(posource)
         print(pofile)
         assert len(pofile.units) == 2
 
-    def test_plurals(self):
+    def test_plurals(self) -> None:
         posource = r"""msgid "Cow"
 msgid_plural "Cows"
 msgstr[0] "Koei"
@@ -320,7 +366,7 @@ msgstr[0] "Sheep"
         assert unit.target == "Sheep"
         assert unit.target.strings == ["Sheep"]
 
-    def test_plural_unicode(self):
+    def test_plural_unicode(self) -> None:
         """Tests that all parts of the multistring are unicode."""
         posource = r"""msgid "Ców"
 msgid_plural "Cóws"
@@ -332,7 +378,7 @@ msgstr[1] "Kóeie"
         assert isinstance(unit.source, multistring)
         assert isinstance(unit.source.strings[1], str)
 
-    def test_nongettext_location(self):
+    def test_nongettext_location(self) -> None:
         """Test that we correctly handle a non-gettext (file:linenumber) location."""
         posource = (
             '#: programming/C/programming.xml:44(para)\nmsgid "test"\nmsgstr "rest"\n'
@@ -346,7 +392,7 @@ msgstr[1] "Kóeie"
         assert locations[0] == "programming/C/programming.xml:44(para)"
         assert isinstance(locations[0], str)
 
-    def test_percent_location(self):
+    def test_percent_location(self) -> None:
         """Test that we correctly handle a location with percent chars."""
         posource = (
             '#: /foo/bar/%%var%%www%%about.html:44\nmsgid "test"\nmsgstr "rest"\n'
@@ -360,7 +406,7 @@ msgstr[1] "Kóeie"
         assert locations[0] == "/foo/bar/%%var%%www%%about.html:44"
 
     @mark.xfail(reason="Not Implemented")
-    def test_kde_plurals(self):
+    def test_kde_plurals(self) -> None:
         """Tests kde-style plurals. (Bug: 191)."""
         posource = r"""msgid "_n Singular\n"
 "Plural"
@@ -381,7 +427,7 @@ msgstr "Een\n"
         assert unit.target == "Een"
         assert unit.target.strings == ["Een", "Twee", "Drie"]
 
-    def test_empty_lines_notes(self):
+    def test_empty_lines_notes(self) -> None:
         """Tests that empty comment lines are preserved."""
         posource = r"""# License name
 #
@@ -394,7 +440,7 @@ msgstr "POT-Creation-Date: 2006-03-08 17:30+0200\n"
         pofile = self.poparse(posource)
         assert bytes(pofile).decode("utf-8") == posource
 
-    def test_fuzzy(self):
+    def test_fuzzy(self) -> None:
         """Checks that fuzzy functionality works as expected."""
         posource = '#, fuzzy\nmsgid "ball"\nmsgstr "bal"\n'
         expectednonfuzzy = 'msgid "ball"\nmsgstr "bal"\n'
@@ -438,7 +484,7 @@ msgstr "POT-Creation-Date: 2006-03-08 17:30+0200\n"
         assert bytes(pofile).decode("utf-8") == expectedfuzzyagain
 
     @mark.xfail(reason="Check differing behaviours between pypo and cpo")
-    def test_makeobsolete_untranslated(self):
+    def test_makeobsolete_untranslated(self) -> None:
         """Tests making an untranslated unit obsolete."""
         posource = '#. The automatic one\n#: test.c\nmsgid "test"\nmsgstr ""\n'
         pofile = self.poparse(posource)
@@ -449,14 +495,14 @@ msgstr "POT-Creation-Date: 2006-03-08 17:30+0200\n"
         assert str(unit) == ""
         # a better way might be for pomerge/pot2po to remove the unit
 
-    def test_merging_automaticcomments(self):
+    def test_merging_automaticcomments(self) -> None:
         """Checks that new automatic comments override old ones."""
         oldsource = '#. old comment\n#: line:10\nmsgid "One"\nmsgstr "Een"\n'
         newsource = '#. new comment\n#: line:10\nmsgid "One"\nmsgstr ""\n'
         expected = '#. new comment\n#: line:10\nmsgid "One"\nmsgstr "Een"\n'
         assert self.pomerge(newsource, oldsource, authoritative=True) == expected
 
-    def test_malformed_units(self):
+    def test_malformed_units(self) -> None:
         """Test that we handle malformed units reasonably."""
         posource = (
             'msgid "thing\nmsgstr "ding"\nmsgid "Second thing"\nmsgstr "Tweede ding"\n'
@@ -464,7 +510,34 @@ msgstr "POT-Creation-Date: 2006-03-08 17:30+0200\n"
         with raises(ValueError):
             self.poparse(posource)
 
-    def test_malformed_obsolete_units(self):
+    def test_invalid_keyword(self) -> None:
+        """
+        Test that invalid/unknown keywords are rejected.
+
+        Regression test for issue where a typo like 'ms' instead of 'msgstr'
+        was silently accepted instead of raising an error. This ensures that
+        PO files with unknown keywords fail validation, similar to msgcat.
+        """
+        # Test case from the original issue: typo 'ms' instead of 'msgstr'
+        posource = """# : projects.py:136
+
+msgid "View"
+ms "Bekyk"
+"""
+        with raises(ValueError):
+            self.poparse(posource)
+
+        # Test other unknown keywords
+        posource = 'msgid "Test"\nmsgxyz "Translation"\n'
+        with raises(ValueError):
+            self.poparse(posource)
+
+        # Test msgstr appearing without msgid
+        posource = 'msgstr "Translation"\n'
+        with raises(ValueError):
+            self.poparse(posource)
+
+    def test_malformed_obsolete_units(self) -> None:
         """Test that we handle malformed obsolete units reasonably."""
         posource = """msgid "thing"
 msgstr "ding"
@@ -477,7 +550,7 @@ msgstr "ding"
         pofile = self.poparse(posource)
         assert len(pofile.units) == 3
 
-    def test_uniforum_po(self):
+    def test_uniforum_po(self) -> None:
         """Test that we handle Uniforum PO files."""
         posource = """# File: ../somefile.cpp, line: 33
 msgid "thing"
@@ -493,7 +566,7 @@ msgstr "tweede"
         # assert pofile.units[0].getlocations() == "File: somefile, line: 300"
         # assert pofile.units[1].getlocations() == "File: anotherfile, line: 200"
 
-    def test_obsolete(self):
+    def test_obsolete(self) -> None:
         """Tests that obsolete messages work."""
         posource = '#~ msgid "Old thing"\n#~ msgstr "Ou ding"\n'
         pofile = self.poparse(posource)
@@ -523,7 +596,7 @@ msgstr "een"
         unit.resurrect()
         assert unit.hasplural()
 
-    def test_obsolete_with_prev_msgid(self):
+    def test_obsolete_with_prev_msgid(self) -> None:
         """Tests that obsolete messages work."""
         # Bug 1429
         # spellchecker:off
@@ -558,7 +631,7 @@ msgstr "een"
         print(bytes(pofile))
         assert bytes(pofile).decode("utf-8") == posource
 
-    def test_header_escapes(self):
+    def test_header_escapes(self) -> None:
         pofile = self.StoreClass()
         pofile.updateheader(
             add=True,
@@ -573,7 +646,7 @@ msgstr "een"
         assert r"\source\core\resource" in pofile.header().target
         assert r"re\\resource" in filecontents
 
-    def test_makeobsolete(self):
+    def test_makeobsolete(self) -> None:
         """Tests making a unit obsolete."""
         posource = '#. The automatic one\n#: test.c\nmsgid "test"\nmsgstr "rest"\n'
         poexpected = '#~ msgid "test"\n#~ msgstr "rest"\n'
@@ -586,7 +659,7 @@ msgstr "een"
         print(pofile)
         assert str(unit) == poexpected
 
-    def test_makeobsolete_plural(self):
+    def test_makeobsolete_plural(self) -> None:
         """Tests making a plural unit obsolete."""
         posource = r"""msgid "Cow"
 msgid_plural "Cows"
@@ -607,7 +680,7 @@ msgstr[1] "Koeie"
         print(pofile)
         assert str(unit) == poexpected
 
-    def test_makeobsolete_msgctxt(self):
+    def test_makeobsolete_msgctxt(self) -> None:
         """Tests making a unit with msgctxt obsolete."""
         posource = '#: test.c\nmsgctxt "Context"\nmsgid "test"\nmsgstr "rest"\n'
         poexpected = '#~ msgctxt "Context"\n#~ msgid "test"\n#~ msgstr "rest"\n'
@@ -622,7 +695,7 @@ msgstr[1] "Koeie"
         print(pofile)
         assert str(unit) == poexpected
 
-    def test_makeobsolete_msgidcomments(self):
+    def test_makeobsolete_msgidcomments(self) -> None:
         """Tests making a unit with msgidcomments obsolete."""
         posource = '#: first.c\nmsgid ""\n"_: first.c\\n"\n"test"\nmsgstr "rest"\n\n#: second.c\nmsgid ""\n"_: second.c\\n"\n"test"\nmsgstr "rest"'
         poexpected = '#~ msgid ""\n#~ "_: first.c\\n"\n#~ "test"\n#~ msgstr "rest"\n'
@@ -636,7 +709,7 @@ msgstr[1] "Koeie"
         print(f"Result:\n{pofile}")
         assert str(unit) == poexpected
 
-    def test_multiline_obsolete(self):
+    def test_multiline_obsolete(self) -> None:
         """Tests for correct output of multiline obsolete messages."""
         posource = '#~ msgid ""\n#~ "Old thing\\n"\n#~ "Second old thing"\n#~ msgstr ""\n#~ "Ou ding\\n"\n#~ "Tweede ou ding"\n'
         pofile = self.poparse(posource)
@@ -648,7 +721,7 @@ msgstr[1] "Koeie"
         print(posource)
         assert bytes(pofile).decode("utf-8") == posource
 
-    def test_merge_duplicates(self):
+    def test_merge_duplicates(self) -> None:
         """Checks that merging duplicates works."""
         posource = '#: source1\nmsgid "test me"\nmsgstr ""\n\n#: source2\nmsgid "test me"\nmsgstr ""\n'
         pofile = self.poparse(posource)
@@ -657,7 +730,7 @@ msgstr[1] "Koeie"
         assert pofile.units[0].getlocations() == ["source1", "source2"]
         print(pofile)
 
-    def test_merge_mixed_sources(self):
+    def test_merge_mixed_sources(self) -> None:
         """Checks that merging works with different source location styles."""
         posource = """
 #: source1
@@ -676,7 +749,7 @@ msgstr ""
         assert len(pofile.units) == 1
         assert pofile.units[0].getlocations() == ["source1", "source2"]
 
-    def test_parse_context(self):
+    def test_parse_context(self) -> None:
         """Tests that msgctxt is parsed correctly and that it is accessible via the api methods."""
         posource = """# Test comment
 #: source1
@@ -700,7 +773,7 @@ msgstr "omskakel"
         assert unit.getcontext() == "verb"
         assert unit.getnotes() == "Test comment 2"
 
-    def test_parse_advanced_context(self):
+    def test_parse_advanced_context(self) -> None:
         """Tests that some weird possible msgctxt scenarios are parsed correctly."""
         posource = r"""# Test multiline context
 #: source1
@@ -742,7 +815,7 @@ msgstr "omskakel"
         )
         assert unit.getnotes() == "Test quotes, newlines and multiline."
 
-    def test_kde_context(self):
+    def test_kde_context(self) -> None:
         """Tests that kde-style msgid comments can be retrieved via getcontext()."""
         posource = r"""# Test comment
 #: source1
@@ -769,7 +842,7 @@ msgstr "omskakel"
         assert unit.getcontext() == "Verb. _: The action of changing."
         assert unit.getnotes() == "Test comment 2"
 
-    def test_broken_kde_context(self):
+    def test_broken_kde_context(self) -> None:
         posource = """msgid "Broken _: here"
 msgstr "Broken _: here"
 """
@@ -778,7 +851,7 @@ msgstr "Broken _: here"
         assert unit.source == "Broken _: here"
         assert unit.target == "Broken _: here"
 
-    def test_id(self):
+    def test_id(self) -> None:
         """Checks that ids work correctly."""
         posource = r"""
 msgid ""
@@ -817,7 +890,7 @@ msgstr[0] ""
 
     #        assert pofile.units[4].getid() == "tree\0trees"
 
-    def test_non_ascii_header_comments(self):
+    def test_non_ascii_header_comments(self) -> None:
         posource = r"""
 # Tëśt þis.
 # Hé Há Hó.
@@ -836,7 +909,7 @@ msgstr "b"
         for line in pofile.units[0].getnotes():
             assert isinstance(line, str)
 
-    def test_non_ascii_header_comments_2(self):
+    def test_non_ascii_header_comments_2(self) -> None:
         posource = r"""
 # Copyright bla.
 msgid ""
@@ -877,7 +950,7 @@ msgstr "b"
         assert "Last-Translator" in header_dict
         assert header_dict["Last-Translator"] == "Tránslátór"
 
-    def test_final_slash(self):
+    def test_final_slash(self) -> None:
         r"""Test that \\ as last character is correctly interpreted (bug 960)."""
         posource = r"""
 msgid ""
@@ -895,7 +968,7 @@ msgstr "プロジェクトが見つかりませんでした"
         print(bytes(pofile2))
         assert bytes(pofile1) == bytes(pofile2)
 
-    def test_unfinished_lines(self):
+    def test_unfinished_lines(self) -> None:
         """Test that we reasonably handle lines with a single quote."""
         posource = r"""
 msgid ""
@@ -910,7 +983,7 @@ msgstr "start thing dingis fish"
         with raises(ValueError):
             self.poparse(posource)
 
-    def test_encoding_change(self):
+    def test_encoding_change(self) -> None:
         posource = r"""
 msgid ""
 msgstr ""
@@ -930,7 +1003,7 @@ msgstr "d"
         assert b'msgstr "\xe1\xb8\x93"' in contents
         assert b"charset=UTF-8" in contents
 
-    def test_istranslated(self):
+    def test_istranslated(self) -> None:
         """Checks that istranslated works ok."""
         posource = r"""
 msgid ""
@@ -951,7 +1024,7 @@ msgstr[0] ""
         assert not unit.istranslated()
         assert unit.get_state_n() == 0
 
-    def test_wrapping(self):
+    def test_wrapping_spaces(self) -> None:
         """This tests that we wrap like gettext."""
         posource = r"""#: file.h:1
 msgid "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345"
@@ -960,6 +1033,7 @@ msgstr "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345"
         # should be unchanged:
         assert self.poreflow(posource) == posource
 
+    def test_wrapping_long(self) -> None:
         posource = r"""#: 2
 msgid "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1"
 msgstr "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1"
@@ -972,6 +1046,7 @@ msgstr ""
 """
         assert self.poreflow(posource) == posource_wanted
 
+    def test_wrapping_long_fit(self) -> None:
         posource = r"""#: 7
 msgid "bla\t12345 12345 12345 12345 12345 12 12345 12345 12345 12345 12345 12345 123"
 msgstr "bla\t12345 12345 12345 12345 12345 15 12345 12345 12345 12345 12345 12345 123"
@@ -984,6 +1059,7 @@ msgstr ""
 """
         assert self.poreflow(posource) == posource_wanted
 
+    def test_wrapping_long_overflow(self) -> None:
         posource = r"""#: 7
 msgid "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1"
 msgstr "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1"
@@ -998,6 +1074,7 @@ msgstr ""
 """
         assert self.poreflow(posource) == posource_wanted
 
+    def test_wrapping_long_multiline(self) -> None:
         posource = r"""#: 8
 msgid "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1234\n1234"
 msgstr "bla\t12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 12345 1234\n1234"
@@ -1026,6 +1103,7 @@ msgstr ""
 """
         assert self.poreflow(posource) == posource_wanted
 
+    def test_wrapping_long_escapes(self) -> None:
         posource = r"""#: 10
 msgid "\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"
 msgstr "\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\"
@@ -1040,7 +1118,7 @@ msgstr ""
 """
         assert self.poreflow(posource) == posource_wanted
 
-    def test_wrapping_cjk(self):
+    def test_wrapping_cjk(self) -> None:
         posource = r"""msgid ""
 msgstr "Content-Type: text/plain; charset=utf-8\n"
 
@@ -1051,7 +1129,7 @@ msgstr ""
 """
         assert self.poreflow(posource) == posource
 
-    def test_wrap_emoji(self):
+    def test_wrap_emoji(self) -> None:
         posource = r"""msgid ""
 msgstr "Content-Type: text/plain; charset=utf-8\n"
 
@@ -1061,7 +1139,7 @@ msgstr ""
 """
         assert self.poreflow(posource) == posource
 
-    def test_wrap_parenthesis(self):
+    def test_wrap_parenthesis(self) -> None:
         posource = r"""msgid ""
 msgstr "Content-Type: text/plain; charset=utf-8\n"
 
@@ -1077,7 +1155,50 @@ msgstr ""
 """
         assert self.poreflow(posource) == posource
 
-    def test_wrap_parenthesis_long(self):
+    def test_wrap_escape_line(self) -> None:
+        gettext_0_22 = r"""msgid ""
+msgstr "Content-Type: text/plain; charset=utf-8\n"
+
+msgid "test"
+msgstr ""
+"%{src?%{dest?转发:入站}:出站} %{ipv6?%{ipv4?<var>IPv4</var> and <var>IPv6</"
+"var>:<var>IPv6</var>}:<var>IPv4</var>}%{proto?, 协议 "
+"%{proto#%{next?, }%{item.types?<var class=\"cbi-tooltip-container\">%{item."
+"name}<span class=\"cbi-tooltip\">具有类型 %{item.types#%{next?, }<var>%{item}"
+"</var>} 的 ICMP</span></var>:<var>%{item.name}</var>}}}%{mark?, 标记 "
+"<var%{mark.inv? data-tooltip=\"匹配除 %{mark.num}%{mark.mask? 带有掩码 "
+"%{mark.mask}} 的 fwmarks。\":%{mark.mask? data-tooltip=\"在比较前对fwmark 应"
+"用掩码 %{mark.mask} 。\"}}>%{mark.val}</var>}%{dscp?, DSCP %{dscp.inv?<var "
+"data-tooltip=\"匹配除 %{dscp.num?:%{dscp.name}} 以外的 DSCP 类型。\">%{dscp."
+"val}</var>:<var>%{dscp.val}</var>}}%{helper?, 助手 %{helper.inv?<var data-"
+"tooltip=\"匹配除 &quot;%{helper.name}&quot; 以外的任意助手。\">%{helper.val}"
+"</var>:<var data-tooltip=\"%{helper.name}\">%{helper.val}</var>}}"
+"""
+        gettext_0_23 = r"""msgid ""
+msgstr "Content-Type: text/plain; charset=utf-8\n"
+
+msgid "test"
+msgstr ""
+"%{src?%{dest?转发:入站}:出站} %{ipv6?%{ipv4?<var>IPv4</var> and <var>IPv6</"
+"var>:<var>IPv6</var>}:<var>IPv4</var>}%{proto?, 协议 %{proto#%{next?, }%"
+"{item.types?<var class=\"cbi-tooltip-container\">%{item.name}<span "
+"class=\"cbi-tooltip\">具有类型 %{item.types#%{next?, }<var>%{item}</var>} 的 "
+"ICMP</span></var>:<var>%{item.name}</var>}}}%{mark?, 标记 <var%{mark.inv? "
+"data-tooltip=\"匹配除 %{mark.num}%{mark.mask? 带有掩码 %{mark.mask}} 的 "
+"fwmarks。\":%{mark.mask? data-tooltip=\"在比较前对fwmark 应用掩码 %"
+"{mark.mask} 。\"}}>%{mark.val}</var>}%{dscp?, DSCP %{dscp.inv?<var data-"
+"tooltip=\"匹配除 %{dscp.num?:%{dscp.name}} 以外的 DSCP 类型。\">%{dscp.val}</"
+"var>:<var>%{dscp.val}</var>}}%{helper?, 助手 %{helper.inv?<var data-"
+"tooltip=\"匹配除 &quot;%{helper.name}&quot; 以外的任意助手。\">%{helper.val}"
+"</var>:<var data-tooltip=\"%{helper.name}\">%{helper.val}</var>}}"
+"""
+        posource = self._get_expected_for_gettext_version(
+            gettext_0_22=gettext_0_22,
+            gettext_0_23=gettext_0_23,
+        )
+        assert self.poreflow(posource) == posource
+
+    def test_wrap_parenthesis_long(self) -> None:
         gettext_0_22 = r"""msgid "test3"
 msgstr ""
 "Must be required by a NotificationListenerService, to ensure that only the "
@@ -1094,25 +1215,14 @@ msgstr ""
 "Manifest.permission#BIND_NOTIFICATION_LISTENER_SERVICE]BIND_NOTIFICATION_LISTENER_SERVICE[/"
 "url]."
 """
-        if issubclass(self.StoreClass, pypo.pofile):
-            # Python wrapper should follow the latest gettext
-            expected = gettext_0_23
-        else:
-            # Choose matching output depending on gettext version
-            from translate.storage.cpo import get_libgettextpo_version
-
-            version = get_libgettextpo_version()
-            if version >= (0, 23, 0):
-                print(f"Detected gettext 0.23 or newer ({version})")
-                expected = gettext_0_23
-            else:
-                print(f"Detected gettext 0.22 or older ({version})")
-                expected = gettext_0_22
+        expected = self._get_expected_for_gettext_version(
+            gettext_0_22=gettext_0_22, gettext_0_23=gettext_0_23
+        )
 
         assert self.poreflow(gettext_0_22) == expected
         assert self.poreflow(gettext_0_23) == expected
 
-    def test_wrap_gettext(self):
+    def test_wrap_gettext(self) -> None:
         gettext_0_23 = r"""# Test
 msgid ""
 msgstr ""
@@ -1175,31 +1285,18 @@ msgstr ""
 """
         gettext_0_20 = gettext_0_21.replace('"\n"n%10', 'n"\n"%10')
 
-        if issubclass(self.StoreClass, pypo.pofile):
-            # Python wrapper should follow the latest gettext
-            expected = gettext_0_23
-        else:
-            # Choose matching output depending on gettext version
-            from translate.storage.cpo import get_libgettextpo_version
-
-            version = get_libgettextpo_version()
-            if version >= (0, 23, 0):
-                print(f"Detected gettext 0.23 or newer ({version})")
-                expected = gettext_0_23
-            elif version >= (0, 21, 0):
-                print(f"Detected gettext 0.21 or newer ({version})")
-                expected = gettext_0_21
-            else:
-                print(f"Detected gettext 0.20 or older ({version})")
-                expected = gettext_0_20
+        expected = self._get_expected_for_gettext_version(
+            gettext_0_20=gettext_0_20,
+            gettext_0_21=gettext_0_21,
+            gettext_0_23=gettext_0_23,
+        )
 
         # Verify that any input wraps to the expected output
         assert self.poreflow(gettext_0_21) == expected
         assert self.poreflow(gettext_0_23) == expected
         assert self.poreflow(gettext_0_20) == expected
 
-    @mark.xfail(reason="Incompatible wrapping with gettext, see #5661")
-    def test_wrap_wide_stop(self):
+    def test_wrap_wide_stop(self) -> None:
         posource = r"""msgid ""
 msgstr "Content-Type: text/plain; charset=utf-8\n"
 
@@ -1218,7 +1315,23 @@ msgstr ""
 """
         assert self.poreflow(posource) == posource
 
-    def test_msgidcomments(self):
+    def test_wrap_escape(self) -> None:
+        posource = r"""msgid ""
+msgstr "Content-Type: text/plain; charset=utf-8\n"
+
+#: ../../content/applications/finance/accounting/payments/online.rst:22
+msgid ""
+"By default, \":doc:`Wire Transfer </applications/finance/payment_providers/"
+"wire_transfer>`\" is the only payment provider activated, but you still have "
+"to fill out the payment details."
+msgstr ""
+"기본값으로 \":doc:`온라인 이체 </applications/finance/payment_providers/"
+"wire_transfer>`\"만 결제대행업체을 사용하도록 설정되어 있으나, 여기에도 결제 "
+"세부 정보를 입력해야 합니다."
+"""
+        assert self.poreflow(posource) == posource
+
+    def test_msgidcomments(self) -> None:
         posource = r"""
 msgid ""
 msgstr ""
@@ -1239,7 +1352,7 @@ msgstr ""
         assert unit.source == "The actual source text"
         assert unit.target.startswith("_: ")
 
-    def test_unicode_ids(self):
+    def test_unicode_ids(self) -> None:
         posource = b"""
 msgid ""
 msgstr ""
@@ -1257,7 +1370,7 @@ msgstr ""
         unit = pofile.units[1]
         assert unit.source == "Raphaël"
 
-    def test_syntax_error(self):
+    def test_syntax_error(self) -> None:
         posource = b"""
 #| identified as a comment
 #|raise an infinite loop bug!
@@ -1267,14 +1380,32 @@ msgstr "texte"
         with raises(ValueError):
             self.poparse(posource)
 
-    def test_invalid(self):
+    def test_invalid(self) -> None:
         posource = b"""
 msg
 """
         with raises(ValueError):
             self.poparse(posource)
 
-    def test_wrapped_msgid(self):
+    def test_c_style_comment_error_reporting(self) -> None:
+        """Test that C-style comments in msgstr show the correct error line."""
+        # C-style comments are not valid in PO files and should trigger an error
+        # pointing to the comment line, not the previous msgstr line
+        posource = b"""msgid "test"
+msgstr ""
+/* option-name: wrap */
+"translation"
+"""
+        with raises(ValueError) as exc_info:
+            self.poparse(posource)
+        error_msg = str(exc_info.value)
+
+        # Verify the error doesn't incorrectly point to msgstr as the problem
+        # The Python parser shows the C-style comment line content
+        # The native gettext parser uses a different format but also shouldn't blame msgstr
+        assert 'msgstr ""' not in error_msg
+
+    def test_wrapped_msgid(self) -> None:
         posource = b"""
 #| msgid "some"
 #|"old text"
@@ -1288,7 +1419,7 @@ msgstr "texte"
             assert unit.prev_source == "someold text"
         assert unit.source == "text"
 
-    def test_missing_plural(self):
+    def test_missing_plural(self) -> None:
         posource = b"""
 msgid "text"
 msgid_plural "texts"
@@ -1296,3 +1427,42 @@ msgstr "texte"
 """
         with raises(ValueError):
             self.poparse(posource)
+
+    def test_unusual_line_endings(self) -> None:
+        r"""Test that files with \r\r\n line endings can be parsed."""
+        # This file has unusual \r\r\n line endings (double CR before LF)
+        posource = b'msgid ""\r\r\nmsgstr ""\r\r\n"Content-Type: text/plain; charset=UTF-8\\n"\r\r\n\r\r\nmsgid "test"\r\r\nmsgstr "rest"\r\r\n'
+        pofile = self.poparse(posource)
+        assert len(pofile.units) == 2
+        assert pofile.units[1].source == "test"
+        assert pofile.units[1].target == "rest"
+
+    def test_charset_with_space(self) -> None:
+        """Test that charset detection works with space after equals sign."""
+        # This file has "charset= koi8-r" with a space after =
+        posource = b'msgid ""\nmsgstr ""\n"Content-Type: text/plain; charset= UTF-8\\n"\n\nmsgid "test"\nmsgstr "rest"\n'
+        pofile = self.poparse(posource)
+        assert pofile.encoding == "UTF-8"
+        assert len(pofile.units) == 2
+        assert pofile.units[1].source == "test"
+
+    def test_duplicate_msgid_error(self) -> None:
+        """Test that duplicate msgid entries (without msgstr) raise an error."""
+        # This file has two consecutive msgid lines without msgstr between them
+        posource = b"""
+msgid ""
+msgstr ""
+"Content-Type: text/plain; charset=UTF-8\\n"
+
+msgid "test"
+msgstr "rest"
+
+msgid "V"
+msgid "V"
+
+msgid "next"
+msgstr "next translation"
+"""
+        # This should raise an error because it's genuinely invalid
+        with raises(ValueError):
+            self.poparse(posource)
