@@ -121,7 +121,7 @@ def unquotefromandroid(source):
     return value.replace('\\"', '"')  # This converts \&quot; to ".
 
 
-_DTD_CODEPOINT2NAME = {
+_DTD_CODEPOINT2NAME: dict[int, str] = {
     ord("%"): "#037",  # Always escape % sign as &#037;.
     ord("&"): "amp",
     # ord("<"): "lt",  # Not really so useful.
@@ -136,11 +136,11 @@ def quotefordtd(source):
         source = source.replace("'", "&apos;")  # This seems not to run.
         if '="' not in source:  # Avoid escaping " chars in href attributes.
             source = source.replace('"', "&quot;")
-            value = '"' + source + '"'  # Quote using double quotes.
+            value = f'"{source}"'  # Quote using double quotes.
         else:
-            value = "'" + source + "'"  # Quote using single quotes.
+            value = f"'{source}'"  # Quote using single quotes.
     else:
-        value = '"' + source + '"'  # Quote using double quotes.
+        value = f'"{source}"'  # Quote using double quotes.
     return value
 
 
@@ -174,7 +174,7 @@ def unquotefromdtd(source):
     return quote.entitydecode(extracted, _DTD_NAME2CODEPOINT)
 
 
-def removeinvalidamps(name, value):
+def removeinvalidamps(name: str, value: str) -> str:
     """
     Find and remove ampersands that are not part of an entity definition.
 
@@ -184,12 +184,8 @@ def removeinvalidamps(name, value):
     down the problem is very difficult, thus by removing potential broken
     ampersand and warning the users we can ensure that the output DTD will
     always be parsable.
-
-    :type name: String
     :param name: Entity name
-    :type value: String
     :param value: Entity text value
-    :rtype: String
     :return: Entity value without bad ampersands
     """
 
@@ -219,7 +215,7 @@ def removeinvalidamps(name, value):
 class dtdunit(base.TranslationUnit):
     """An entity definition from a DTD file (and any associated comments)."""
 
-    def __init__(self, source="", android=False):
+    def __init__(self, source="", android=False) -> None:
         """Construct the dtdunit, prepare it for parsing."""
         self.android = android
 
@@ -243,7 +239,7 @@ class dtdunit(base.TranslationUnit):
         return unquotefromdtd(self.definition)
 
     @source.setter
-    def source(self, source):
+    def source(self, source) -> None:
         """Sets the definition to the quoted value of source."""
         if self.android:
             self.definition = quoteforandroid(source)
@@ -259,7 +255,7 @@ class dtdunit(base.TranslationUnit):
         return unquotefromdtd(self.definition)
 
     @target.setter
-    def target(self, target):
+    def target(self, target) -> None:
         """Sets the definition to the quoted value of target."""
         if target is None:
             target = ""
@@ -272,7 +268,7 @@ class dtdunit(base.TranslationUnit):
     def getid(self):
         return self.entity
 
-    def setid(self, new_id):
+    def setid(self, new_id) -> None:  # ty:ignore[invalid-method-override]
         self.entity = new_id
 
     def getlocations(self):
@@ -280,7 +276,7 @@ class dtdunit(base.TranslationUnit):
         assert quote.rstripeol(self.entity) == self.entity
         return [self.entity]
 
-    def addlocation(self, location):
+    def addlocation(self, location) -> None:
         """Set the entity to the given "location"."""
         self.entity = location
 
@@ -293,204 +289,7 @@ class dtdunit(base.TranslationUnit):
     def istranslatable(self):
         return getattr(self, "entityparameter", None) != "SYSTEM" and not self.isblank()
 
-    def parse(self, dtdsrc):
-        """Read the first dtd element from the source code into this object, return linesprocessed."""
-        self.comments = []
-        # make all the lists the same
-        self._locfilenotes = self.comments
-        self._locgroupstarts = self.comments
-        self._locgroupends = self.comments
-        self._locnotes = self.comments
-        # self._locfilenotes = []
-        # self._locgroupstarts = []
-        # self._locgroupends = []
-        # self._locnotes = []
-        # self.comments = []
-        self.entity = None
-        self.definition = ""
-        if not dtdsrc:
-            return 0
-        lines = dtdsrc.split("\n")
-        linesprocessed = 0
-        comment = ""
-        for line in lines:
-            line += "\n"
-            linesprocessed += 1
-            if not self.incomment:
-                if line.find("<!--") != -1:
-                    self.incomment = True
-                    self.continuecomment = False
-                    # now work out the type of comment, and save it (remember we're not in the comment yet)
-                    comment, _dummy = quote.extract(line, "<!--", "-->", None, 0)
-                    if comment.find("LOCALIZATION NOTE") != -1:
-                        l = quote.findend(comment, "LOCALIZATION NOTE")
-                        while comment[l] == " ":
-                            l += 1
-                        if comment.find("FILE", l) == l:
-                            self.commenttype = "locfile"
-                        elif comment.find("BEGIN", l) == l:
-                            self.commenttype = "locgroupstart"
-                        elif comment.find("END", l) == l:
-                            self.commenttype = "locgroupend"
-                        else:
-                            self.commenttype = "locnote"
-                    else:
-                        # plain comment
-                        self.commenttype = "comment"
-                # FIXME: bloody entity might share a line with something important
-                elif not self.inentity and re.search(r"%[^;%]+;", line):
-                    # now work out the type of comment, and save it (remember we're not in the comment yet)
-                    self.comments.append(("comment", line))
-                    line = ""
-                    continue
-
-            if self.incomment:
-                # some kind of comment
-                (comment, self.incomment) = quote.extract(
-                    line, "<!--", "-->", None, self.continuecomment
-                )
-                self.continuecomment = self.incomment
-                # strip the comment out of what will be parsed
-                line = line.replace(comment, "", 1)
-                # add a end of line of this is the end of the comment
-                if not self.incomment:
-                    if line.isspace():
-                        comment += line
-                        line = ""
-                    else:
-                        comment += "\n"
-                # check if there's actually an entity definition that's commented out
-                # TODO: parse these, store as obsolete messages
-                # if comment.find('<!ENTITY') != -1:
-                #     # remove the entity from the comment
-                #     comment, dummy = quote.extractwithoutquotes(comment, ">", "<!ENTITY", None, 1)
-                # depending on the type of comment (worked out at the start), put it in the right place
-                # make it record the comment and type as a tuple
-                commentpair = (self.commenttype, comment)
-                if self.commenttype == "locfile":
-                    self._locfilenotes.append(commentpair)
-                elif self.commenttype == "locgroupstart":
-                    self._locgroupstarts.append(commentpair)
-                elif self.commenttype == "locgroupend":
-                    self._locgroupends.append(commentpair)
-                elif self.commenttype == "locnote":
-                    self._locnotes.append(commentpair)
-                elif self.commenttype == "comment":
-                    self.comments.append(commentpair)
-
-            if not self.inentity and not self.incomment:
-                entitypos = line.find("<!ENTITY")
-                if entitypos != -1:
-                    self.inentity = True
-                    beforeentity = line[:entitypos].strip()
-                    if beforeentity.startswith("#"):
-                        self.hashprefix = beforeentity
-                    self.entitypart = "start"
-                else:
-                    self.unparsedlines.append(line)
-
-            if self.inentity:
-                if self.entitypart == "start":
-                    # the entity definition
-                    e = quote.findend(line, "<!ENTITY")
-                    line = line[e:]
-                    self.entitypart = "name"
-                    self.entitytype = "internal"
-                if self.entitypart == "name":
-                    s = 0
-                    e = 0
-                    while e < len(line) and line[e].isspace():
-                        e += 1
-                    self.space_pre_entity = " " * (e - s)
-                    s = e
-                    self.entity = ""
-                    if e < len(line) and line[e] == "%":
-                        self.entitytype = "external"
-                        self.entityparameter = ""
-                        e += 1
-                        while e < len(line) and line[e].isspace():
-                            e += 1
-                    while e < len(line) and not line[e].isspace():
-                        self.entity += line[e]
-                        e += 1
-                    s = e
-
-                    assert quote.rstripeol(self.entity) == self.entity
-                    while e < len(line) and line[e].isspace():
-                        e += 1
-                    self.space_pre_definition = " " * (e - s)
-                    if self.entity:
-                        if self.entitytype == "external":
-                            self.entitypart = "parameter"
-                        else:
-                            self.entitypart = "definition"
-                        # remember the start position and the quote character
-                        if e == len(line):
-                            self.entityhelp = None
-                            e = 0
-                            continue
-                        if self.entitypart == "definition":
-                            self.entityhelp = (e, line[e])
-                            self.instring = False
-                if self.entitypart == "parameter":
-                    while e < len(line) and line[e].isspace():
-                        e += 1
-                    paramstart = e
-                    while e < len(line) and line[e].isalnum():
-                        e += 1
-                    self.entityparameter += line[paramstart:e]
-                    while e < len(line) and line[e].isspace():
-                        e += 1
-                    line = line[e:]
-                    e = 0
-                    if not line:
-                        continue
-                    if line[0] in {'"', "'"}:
-                        self.entitypart = "definition"
-                        self.entityhelp = (e, line[e])
-                        self.instring = False
-                if self.entitypart == "definition":
-                    if self.entityhelp is None:
-                        e = 0
-                        while e < len(line) and line[e].isspace():
-                            e += 1
-                        if e == len(line):
-                            continue
-                        self.entityhelp = (e, line[e])
-                        self.instring = False
-                    # actually the lines below should remember instring, rather than using it as dummy
-                    e = self.entityhelp[0]
-                    if self.entityhelp[1] == "'":
-                        (defpart, self.instring) = quote.extract(
-                            line[e:],
-                            "'",
-                            "'",
-                            startinstring=self.instring,
-                            allowreentry=False,
-                        )
-                    elif self.entityhelp[1] == '"':
-                        (defpart, self.instring) = quote.extract(
-                            line[e:],
-                            '"',
-                            '"',
-                            startinstring=self.instring,
-                            allowreentry=False,
-                        )
-                    else:
-                        raise ValueError(
-                            f"Unexpected quote character... {self.entityhelp[1]!r}"
-                        )
-                    # for any following lines, start at the beginning of the line. remember the quote character
-                    self.entityhelp = (0, self.entityhelp[1])
-                    self.definition += defpart
-                    if not self.instring:
-                        self.closing = line[e + len(defpart) :].rstrip("\n\r")
-                        self.inentity = False
-                        break
-
-        return linesprocessed
-
-    def __str__(self):
+    def __str__(self) -> str:
         """Convert to a string."""
         return self.getoutput()
 
@@ -501,34 +300,19 @@ class dtdunit(base.TranslationUnit):
         lines.extend(self.unparsedlines)
         if self.isblank():
             result = "".join(lines)
-            return result.rstrip() + "\n"
+            return f"{result.rstrip()}\n"
         # for f in self._locfilenotes: yield f
         # for ge in self._locgroupends: yield ge
         # for gs in self._locgroupstarts: yield gs
         # for n in self._locnotes: yield n
         if len(self.entity) > 0:
             if getattr(self, "entitytype", None) == "external":
-                entityline = (
-                    "<!ENTITY % "
-                    + self.entity
-                    + " "
-                    + self.entityparameter
-                    + " "
-                    + self.definition
-                    + self.closing
-                )
+                entityline = f"<!ENTITY % {self.entity} {self.entityparameter} {self.definition}{self.closing}"  # ty:ignore[unresolved-attribute]
             else:
-                entityline = (
-                    "<!ENTITY"
-                    + self.space_pre_entity
-                    + self.entity
-                    + self.space_pre_definition
-                    + self.definition
-                    + self.closing
-                )
+                entityline = f"<!ENTITY{self.space_pre_entity}{self.entity}{self.space_pre_definition}{self.definition}{self.closing}"
             if getattr(self, "hashprefix", None):
-                entityline = self.hashprefix + " " + entityline
-            lines.append(entityline + "\n")
+                entityline = f"{self.hashprefix} {entityline}"  # ty:ignore[unresolved-attribute]
+            lines.append(f"{entityline}\n")
         return "".join(lines)
 
 
@@ -537,7 +321,7 @@ class dtdfile(base.TranslationStore):
 
     UnitClass = dtdunit
 
-    def __init__(self, inputfile=None, android=False):
+    def __init__(self, inputfile=None, android=False) -> None:
         """Construct a dtdfile, optionally reading in from inputfile."""
         super().__init__()
         self.filename = getattr(inputfile, "name", "")
@@ -546,44 +330,299 @@ class dtdfile(base.TranslationStore):
             dtdsrc = inputfile.read()
             self.parse(dtdsrc)
 
-    def parse(self, dtdsrc):
+    def _determine_comment_type(self, comment: str) -> str:
+        """Determine the type of a DTD comment."""
+        if comment.find("LOCALIZATION NOTE") != -1:
+            pos = quote.findend(comment, "LOCALIZATION NOTE")
+            while pos < len(comment) and comment[pos] == " ":
+                pos += 1
+            if comment.find("FILE", pos) == pos:
+                return "locfile"
+            if comment.find("BEGIN", pos) == pos:
+                return "locgroupstart"
+            if comment.find("END", pos) == pos:
+                return "locgroupend"
+            return "locnote"
+        return "comment"
+
+    def _store_comment(self, unit: dtdunit, commenttype: str, comment: str) -> None:
+        """Store a comment in the appropriate list on the unit."""
+        commentpair = (commenttype, comment)
+        comment_targets = {
+            "locfile": unit._locfilenotes,  # ty:ignore[unresolved-attribute]
+            "locgroupstart": unit._locgroupstarts,  # ty:ignore[unresolved-attribute]
+            "locgroupend": unit._locgroupends,  # ty:ignore[unresolved-attribute]
+            "locnote": unit._locnotes,  # ty:ignore[unresolved-attribute]
+            "comment": unit.comments,
+        }
+        comment_targets.get(commenttype, unit.comments).append(commentpair)
+
+    def _parse_entity_name(self, line: str) -> tuple[str, str, str, str, int]:
+        """
+        Parse entity name from line and return entity info.
+
+        Returns:
+            tuple: (entity_name, entitytype, space_pre_entity, space_pre_definition, position)
+
+        """
+        e = 0
+        while e < len(line) and line[e].isspace():
+            e += 1
+        space_pre_entity = " " * e
+        entity_name = ""
+        entitytype = "internal"
+
+        if e < len(line) and line[e] == "%":
+            entitytype = "external"
+            e += 1
+            while e < len(line) and line[e].isspace():
+                e += 1
+
+        while e < len(line) and not line[e].isspace():
+            entity_name += line[e]
+            e += 1
+        s = e
+
+        while e < len(line) and line[e].isspace():
+            e += 1
+        space_pre_definition = " " * (e - s)
+
+        return entity_name, entitytype, space_pre_entity, space_pre_definition, e
+
+    def _extract_entity_definition(
+        self, line: str, entityhelp: tuple[int, str], instring: bool
+    ) -> tuple[str, bool]:
+        """
+        Extract entity definition from line.
+
+        Returns:
+            tuple: (definition_part, still_in_string) or raises ValueError
+
+        """
+        e = entityhelp[0]
+        quote_char = entityhelp[1]
+
+        if quote_char == "'":
+            return quote.extract(
+                line[e:], "'", "'", startinstring=instring, allowreentry=False
+            )
+        if quote_char == '"':
+            return quote.extract(
+                line[e:], '"', '"', startinstring=instring, allowreentry=False
+            )
+        raise ValueError(f"Unexpected quote character... {quote_char!r}")
+
+    def parse(self, dtdsrc) -> None:  # ty:ignore[invalid-method-override]
         """Read the source code of a dtd file in and include them as dtdunits in self.units."""
-        start = 0
-        end = 0
-        lines = dtdsrc.split(b"\n")
-        while end < len(lines):
-            if start == end:
-                end += 1
-            foundentity = False
-            while end < len(lines):
-                if end >= len(lines):
-                    break
-                if lines[end].find(b"<!ENTITY") > -1:
-                    foundentity = True
-                if foundentity and end_entity_re.match(lines[end]):
-                    end += 1
-                    break
-                end += 1
+        if not dtdsrc:
+            return
 
-            linesprocessed = 1  # to initialise loop
-            while linesprocessed >= 1:
-                newdtd = dtdunit(android=self.android)
-                try:
-                    linesprocessed = newdtd.parse(
-                        (b"\n".join(lines[start:end])).decode(self.encoding)
-                    )
-                    if linesprocessed >= 1 and (
-                        not newdtd.isblank() or newdtd.unparsedlines
-                    ):
-                        self.units.append(newdtd)
-                except Exception as e:
-                    warnings.warn(
-                        "%s\nError occurred between lines %d and %d:\n%s"
-                        % (e, start + 1, end, b"\n".join(lines[start:end]))
-                    )
-                start += linesprocessed
+        # Decode the source
+        source = dtdsrc.decode(self.encoding)
+        lines = source.split("\n")
 
-    def serialize(self, out):
+        # When splitting by "\n", a trailing newline creates an empty string at the end.
+        # For example: "text\n" → ["text", ""]
+        # We remove this empty element to avoid creating an unwanted blank unit.
+        # Intentional blank lines (e.g., "text\n\n") still work correctly:
+        # "text\n\n" → ["text", "", ""] → ["text", ""] (one blank line preserved)
+        if lines and not lines[-1]:
+            lines = lines[:-1]
+
+        # Parse state
+        line_idx = 0
+
+        while line_idx < len(lines):
+            # Create a new unit
+            newdtd = dtdunit(android=self.android)
+
+            # Initialize unit state
+            newdtd.comments = []
+            newdtd._locfilenotes = newdtd.comments  # ty:ignore[unresolved-attribute]
+            newdtd._locgroupstarts = newdtd.comments  # ty:ignore[unresolved-attribute]
+            newdtd._locgroupends = newdtd.comments  # ty:ignore[unresolved-attribute]
+            newdtd._locnotes = newdtd.comments  # ty:ignore[unresolved-attribute]
+            newdtd.entity = None
+            newdtd.definition = ""
+            newdtd.unparsedlines = []
+
+            # Parsing state variables
+            incomment = False
+            continuecomment = False
+            inentity = False
+            commenttype = "comment"
+            entitypart = None
+            entitytype = "internal"
+            entityhelp = None
+            instring = False
+            space_pre_entity = ""
+            space_pre_definition = ""
+
+            has_content = False  # Track if this unit has any content
+            malformed = False  # Track if this unit is malformed
+
+            # Parse lines until we have a complete unit or find the start of the next one
+            while line_idx < len(lines):
+                line = f"{lines[line_idx]}\n"
+
+                if not incomment:
+                    if line.find("<!--") != -1:
+                        incomment = True
+                        continuecomment = False
+                        has_content = True
+                        # Work out the type of comment
+                        comment, _dummy = quote.extract(line, "<!--", "-->", None, 0)
+                        commenttype = self._determine_comment_type(comment)
+                    elif not inentity and re.search(r"%[^;%]+;", line):
+                        # Entity reference line
+                        newdtd.comments.append(("comment", line))
+                        has_content = True
+                        line = ""
+                        line_idx += 1
+                        continue
+
+                if incomment:
+                    # Parse comment
+                    (comment, incomment) = quote.extract(
+                        line, "<!--", "-->", None, continuecomment
+                    )
+                    continuecomment = incomment
+                    # Strip the comment out of what will be parsed
+                    line = line.replace(comment, "", 1)
+                    # Add an end of line if this is the end of the comment
+                    if not incomment:
+                        if line.isspace():
+                            comment += line
+                            line = ""
+                        else:
+                            comment += "\n"
+                    # Store the comment
+                    self._store_comment(newdtd, commenttype, comment)
+
+                if not inentity and not incomment:
+                    entitypos = line.find("<!ENTITY")
+                    if entitypos != -1:
+                        inentity = True
+                        has_content = True
+                        beforeentity = line[:entitypos].strip()
+                        if beforeentity.startswith("#"):
+                            newdtd.hashprefix = beforeentity  # ty:ignore[unresolved-attribute]
+                        entitypart = "start"
+                    else:
+                        # Add to unparsed lines
+                        newdtd.unparsedlines.append(line)
+                        if not line.isspace():
+                            has_content = True
+
+                if inentity:
+                    if entitypart == "start":
+                        # The entity definition
+                        e = quote.findend(line, "<!ENTITY")
+                        line = line[e:]
+                        entitypart = "name"
+
+                    if entitypart == "name":
+                        (
+                            entity_name,
+                            entitytype,
+                            space_pre_entity,
+                            space_pre_definition,
+                            e,
+                        ) = self._parse_entity_name(line)
+
+                        newdtd.entity = entity_name
+                        assert quote.rstripeol(entity_name) == entity_name
+                        if newdtd.entity:
+                            newdtd.entitytype = entitytype  # ty:ignore[unresolved-attribute]
+                            if entitytype == "external":
+                                entitypart = "parameter"
+                                newdtd.entityparameter = ""  # ty:ignore[unresolved-attribute]
+                            else:
+                                entitypart = "definition"
+                            # Remember the start position and the quote character
+                            if e == len(line):
+                                entityhelp = None
+                                e = 0
+                                line_idx += 1
+                                continue
+                            if entitypart == "definition":
+                                entityhelp = (e, line[e])
+                                instring = False
+
+                    if entitypart == "parameter":
+                        while e < len(line) and line[e].isspace():
+                            e += 1
+                        paramstart = e
+                        while e < len(line) and line[e].isalnum():
+                            e += 1
+                        newdtd.entityparameter += line[paramstart:e]  # ty:ignore[unresolved-attribute]
+                        while e < len(line) and line[e].isspace():
+                            e += 1
+                        line = line[e:]
+                        e = 0
+                        if not line:
+                            line_idx += 1
+                            continue
+                        if line[0] in {'"', "'"}:
+                            entitypart = "definition"
+                            entityhelp = (e, line[e])
+                            instring = False
+
+                    if entitypart == "definition":
+                        if entityhelp is None:
+                            e = 0
+                            while e < len(line) and line[e].isspace():
+                                e += 1
+                            if e == len(line):
+                                line_idx += 1
+                                continue
+                            entityhelp = (e, line[e])
+                            instring = False
+                        # Extract the definition part
+                        e = entityhelp[0]
+                        try:
+                            defpart, instring = self._extract_entity_definition(
+                                line, entityhelp, instring
+                            )
+                        except ValueError as exc:
+                            # Handle malformed entities gracefully
+                            warnings.warn(str(exc))
+                            # Mark as malformed and skip to next line
+                            malformed = True
+                            line_idx += 1
+                            break
+                        # For any following lines, start at the beginning of the line
+                        entityhelp = (0, entityhelp[1])
+                        newdtd.definition += defpart
+                        if not instring:
+                            closing = line[e + len(defpart) :].rstrip("\n\r")
+                            inentity = False
+                            # Entity is complete
+                            newdtd.space_pre_entity = space_pre_entity
+                            newdtd.space_pre_definition = space_pre_definition
+                            newdtd.closing = closing
+                            line_idx += 1
+                            break
+
+                line_idx += 1
+
+                # If we have no entity and no comment in progress and no content, skip this unit
+                if (
+                    not inentity
+                    and not incomment
+                    and not has_content
+                    and line_idx < len(lines)
+                ):
+                    break
+
+            # Add the unit if it's not blank or has unparsed lines or has comments,
+            # but skip malformed entities
+            if not malformed and (
+                not newdtd.isblank() or newdtd.unparsedlines or newdtd.comments
+            ):
+                self.units.append(newdtd)
+
+    def serialize(self, out) -> None:
         """Write content to file."""
         content = b""
         for dtd in self.units:
@@ -594,14 +633,13 @@ class dtdfile(base.TranslationStore):
             warnings.warn(f"DTD file '{self.filename}' does not validate")
             out.truncate(0)
 
-    def _valid_store(self, content):
+    def _valid_store(self, content: bytes) -> bool:
         """
         Validate the store to determine if it is valid.
 
         This uses ElementTree to parse the DTD
 
         :return: If the store passes validation
-        :rtype: Boolean
         """
         # Android files are invalid DTDs
         if not self.android:
